import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {
  AbstractControl,
  FormArray,
  FormControl,
  FormGroup,
  FormsModule,
  ReactiveFormsModule,
  Validators
} from "@angular/forms";
import {Hotel} from "../model/hotel";
import {RouterLink} from "@angular/router";
import {NgForOf} from "@angular/common";
import {debounceTime} from "rxjs";
import {ErrorMsgComponent} from "./error-msg.component";

@Component({
  selector: 'app-edit-hotel',
  standalone: true,
  imports: [
    FormsModule,
    ReactiveFormsModule,
    RouterLink,
    NgForOf,
    ErrorMsgComponent,
  ],
  template: `
    <ng-form [formGroup]="form" (ngSubmit)="submit()">
      <label for="name">Name</label>
      <br>
      <input id="name" type="text" formControlName="name">
      <app-error-msg [msg]="errorMsgs['name']"></app-error-msg>
      <br>
      <label for="description">Description</label>
      <br>
      <input id="description" type="text" formControlName="description">
      <app-error-msg [msg]="errorMsgs['description']"></app-error-msg>
      <br>
      <label for="price">Price</label>
      <br>
      <input id="price" type="number" formControlName="price">
      <app-error-msg [msg]="errorMsgs['price']"></app-error-msg>
      <br>
      <label for="rating">Rating</label>
      <br>
      <input id="rating" type="number" formControlName="rating" min="0" max="5">
      <app-error-msg [msg]="errorMsgs['rating']"></app-error-msg>
      <br>
      <br>
      <button (click)="addTag()">+</button>
      <div formArrayName="tags">
        <div *ngFor="let tag of getTags().controls; let i = index">
          <input type="text" placeholder="tag" [formControlName]="i">
          <button (click)="deleteTag(tag)">-</button>
        </div>
        <app-error-msg [msg]="errorMsgs['tags']"></app-error-msg>
      </div>
      <br>
      <br>
      <button type="submit" (click)="submit()">Submit</button>
    </ng-form>

    <button routerLink="/hotels">back</button>
  `,
})
export class EditHotelComponent implements OnInit {
  errorMsgs: Record<string, string> = {};

  @Input()
  hotel: Hotel | null = null;

  @Output()
  updateHotel: EventEmitter<Hotel> = new EventEmitter

  form!: FormGroup

  validationErrors: Record<string, string> = {
    required: 'This field is required',
    min: 'Value must be greater than or equal to 0',
    max: 'Value must be less than or equal to 5'
  };

  ngOnInit(): void {
    const tags = [];

    for (const tag of this.hotel?.tags ?? []) {
      tags.push(new FormControl(tag, [Validators.required]));
    }

    this.form = new FormGroup({
      name: new FormControl(this.hotel?.hotelName, [Validators.required]),
      description: new FormControl(this.hotel?.description, [Validators.required]),
      price: new FormControl(this.hotel?.price, [Validators.required]),
      rating: new FormControl(this.hotel?.rating, [Validators.required, Validators.min(0), Validators.max(5)]),
      tags: new FormArray(tags),
    })

    Object.keys(this.form.controls).forEach(controlName => {
      const control = this.form.get(controlName);
      if (!control) {
        return;
      }

      control?.valueChanges?.pipe(debounceTime(1000)).subscribe(() => {this.setErrorMessage(controlName, control)});
    });
  }

  submit() {
    if (!this.form.valid) {

    }

    const hotel: Hotel = {
      imageUrl: this.hotel?.imageUrl ?? "",
      hotelName: this.form.value.name,
      description: this.form.value.description,
      price: this.form.value.price,
      rating: this.form.value.rating,
      id: this.hotel?.id ?? 0,
      tags: this.form.value.tags ?? []
    };

    this.updateHotel.emit(hotel)

    this.hotel = hotel;
  }

  getTags() {
    return this.form.controls['tags'] as FormArray;
  }

  addTag() {
    this.getTags().push(new FormControl('', [Validators.required]));
  }

  deleteTag(tagElement: AbstractControl) {
    this.getTags().removeAt(this.getTags().controls.indexOf(tagElement));
  }

  setErrorMessage(controlName: string, control: AbstractControl) {
    this.errorMsgs[controlName] = Object.keys(control.errors ?? {}).map ((key) => this.validationErrors[key]).join(' ');
  }
}
